@@ -526,33 +526,46 @@ DOCUMENTATION__drop_email_account = SwaggerDocumentation(
     )
 )
 
-DOCUMENTATION__drop_databases = SwaggerDocumentation(
+ARG_RES__database_name = SwaggerArgumentResponse(
+    name=KEY__name,
+    description="The name of the database",
+    arg_type=str,
+    example=["invoicing_live"]
+)
+
+DOCUMENTATION__databases = SwaggerDocumentation(
     tags="Database",
-    methods=SwaggerMethod(
-        name="Drop database",
-        description="Drops the database associated with a tenant",
-        method=REST__DELETE,
-        arguments=[
-            SwaggerArgumentResponse(
-                name=KEY__name,
-                description="The name of the database",
-                arg_type=str,
-                example=["invoicing_live"]
-            ),
-            SwaggerArgumentResponse(
-                name="delete_record",
-                description="Whether or not to delete the JAAQL record of this database or just the physical db. Defaults to false",
-                arg_type=bool,
-                required=False,
-                condition="Defaults to false"
-            ),
-            SwaggerArgumentResponse(
-                name="silent",
-                description="Whether or not to return an error when the database doesn't exist",
-                arg_type=bool
-            )
-        ]
-    )
+    methods=[
+        SwaggerMethod(
+            name="Drop database",
+            description="Drops the database associated with a tenant",
+            method=REST__DELETE,
+            arguments=[
+                ARG_RES__database_name,
+                SwaggerArgumentResponse(
+                    name="delete_record",
+                    description="Whether or not to delete the JAAQL record of this database or just the physical db. Defaults to false",
+                    arg_type=bool,
+                    required=False,
+                    condition="Defaults to false"
+                ),
+                SwaggerArgumentResponse(
+                    name="silent",
+                    description="Whether or not to return an error when the database doesn't exist",
+                    arg_type=bool,
+                    required=False
+                )
+            ]
+        ),
+        SwaggerMethod(
+            name="Create database",
+            description="Creates the database associated with a tenant",
+            method=REST__POST,
+            arguments=[
+                ARG_RES__database_name
+            ]
+        )
+    ]
 )
 
 DOCUMENTATION__refresh_app_config = SwaggerDocumentation(
