@@ -1,4 +1,5 @@
 CREATE ROLE jaaql__registered;
+CREATE DOMAIN safe_path AS varchar(255) CHECK (VALUE ~* '^[a-z0-9_\-\/]+$');
 
 -- Install script
 -- (1) Create tables
@@ -47,6 +48,7 @@ grant execute on function get_tenant() to public;
 create table tenant (
     deleted timestamptz,
     name postgres_addressable_name not null,
+    enc_symmetric_key varchar(255) not null,
     primary key (name) );
 -- tenant_database...
 create table tenant_database (
@@ -98,16 +100,14 @@ create table email_account (
 
 create table email_template (
     name varchar(60) NOT NULL,
+    application character varying(63) not null,
     tenant postgres_addressable_name not null,
-    PRIMARY KEY (tenant, name),
+    PRIMARY KEY (tenant, name, application),
     FOREIGN KEY (tenant) REFERENCES tenant,
-    application postgres_addressable_name not null,
-    FOREIGN KEY (tenant, application) REFERENCES application(tenant, name),
-    account varchar(60) NOT NULL,
+    FOREIGN KEY (tenant, application) REFERENCES application(tenant, name) ON UPDATE CASCADE ON DELETE CASCADE,
     subject varchar(255),
-    FOREIGN KEY (tenant, account) REFERENCES email_account(tenant, name),
     description text,
-    app_relative_path postgres_addressable_name,  -- Not a mistake for this domain type. Means app_relative_path cannot be ../../secret. Also secured at application level code in case db is tampered with
+    app_relative_path safe_path,  -- Not a mistake for this domain type. Means app_relative_path cannot be ../../secret. Also secured at application level code in case db is tampered with
     check ((subject is null) = (app_relative_path is null)),
     schema character varying(63),
     FOREIGN KEY (tenant, schema, application) REFERENCES application_schema(tenant, name, application),
@@ -115,27 +115,46 @@ create table email_template (
     data_validation_view postgres_addressable_name,
     CHECK ((data_validation_table is null and data_validation_view is null) or data_validation_table is not null),
     recipient_validation_view postgres_addressable_name,
-    check (schema is null or (recipient_validation_view is null and data_validation_table is null)),
+    check ((recipient_validation_view is null and data_validation_table is null) or schema is not null),
     allow_signup boolean default false not null,
     allow_confirm_signup_attempt boolean default false not null,
     allow_reset_password boolean default false not null,
-    check (allow_signup::int + allow_confirm_signup_attempt::int + allow_reset_password::int < 2)
+    check (allow_signup::int + allow_confirm_signup_attempt::int + allow_reset_password::int < 2),
+    default_email_signup boolean,
+    check (default_email_signup is not false),
+    default_email_already_signed_up boolean,
+    check (default_email_already_signed_up is not false),
+    default_reset_password boolean,
+    check (default_reset_password is not false)
 );
+CREATE UNIQUE INDEX template_unq_signup ON email_template (tenant, name, application, default_email_signup);
+CREATE UNIQUE INDEX template_unq_already_signed_up ON email_template (tenant, name, application, default_email_already_signed_up);
+CREATE UNIQUE INDEX template_unq_reset_password ON email_template (tenant, name, application, default_reset_password);
 
 -- application_configuration...
 create table application_configuration (
     tenant character varying(63) not null,
-    default_email_signup_template varchar(60),
-    default_email_already_signed_up_template varchar(60),
-    default_reset_password_template varchar(60),
     application character varying(63) not null,
-    url varchar(254),
+    url varchar(254) not null,
+    artifact_base_uri varchar(254) not null,  --Used to load templates and renderable documents
     name character varying(63) not null,
-    primary key (tenant, application, name),
-    FOREIGN KEY (default_email_signup_template, tenant) REFERENCES email_template (name, tenant),
-    FOREIGN KEY (default_email_already_signed_up_template, tenant) REFERENCES email_template (name, tenant),
-    FOREIGN KEY (default_reset_password_template, tenant) REFERENCES email_template (name, tenant)
+    primary key (tenant, application, name)
 );
+
+create table email_template_configuration (
+    tenant postgres_addressable_name not null,
+    application character varying(63) not null,
+    configuration character varying(63) not null,
+    template varchar(60) not null,
+    account varchar(60) NOT NULL,
+    override_send_name varchar(255),
+    primary key (tenant, application, configuration, template),
+    FOREIGN KEY (tenant, account) REFERENCES email_account(tenant, name) ON DELETE CASCADE ON UPDATE CASCADE,
+    FOREIGN KEY (tenant, application, template) REFERENCES email_template(tenant, application, name) ON UPDATE CASCADE ON DELETE CASCADE,
+    FOREIGN KEY (tenant, application) REFERENCES application(tenant, name) ON UPDATE CASCADE ON DELETE CASCADE,
+    FOREIGN KEY (tenant, application, configuration) REFERENCES application_configuration(tenant, application, name) ON UPDATE CASCADE ON DELETE CASCADE
+);
+
 -- account...
 create table account (
     tenant character varying(63) not null,
@@ -143,9 +162,10 @@ create table account (
     username character varying(255) not null,
     deleted boolean default false,
     application varchar(63),
+    password uuid,
     configuration varchar(63),
-    FOREIGN KEY (application, tenant) REFERENCES application(name, tenant),
-    FOREIGN KEY (application, tenant, configuration) REFERENCES application_configuration(application, tenant, name),
+    FOREIGN KEY (application, tenant) REFERENCES application(name, tenant) ON DELETE CASCADE,
+    FOREIGN KEY (application, tenant, configuration) REFERENCES application_configuration(application, tenant, name) ON DELETE CASCADE,
     public_password text,
     CHECK ((application is null) = (public_password is null)),
     CHECK((application is null) = (configuration is null)),
@@ -157,16 +177,18 @@ create table account_password (
     account jaaql_account_id not null,
     password_hash character varying(512) not null,
     created timestamp not null default current_timestamp,
-    unique(account, password_hash));
+    unique(account, password_hash)
+);
+
+alter table account add constraint account_current_password__account
+    foreign key (password)
+        references account_password (id);
 
 create view tenant_application_schemas as (
     SELECT
         app.tenant,
         app.name,
         ac.name as configuration_name,
-        ac.default_email_signup_template as configuration_default_email_signup_template,
-        ac.default_email_already_signed_up_template as configuration_default_email_already_signed_up_template,
-        ac.default_reset_password_template as configuration_default_reset_password_template,
         ac.url as configuration_url,
         asch.name as schema_name,
         ad.database as schema_database,
@@ -179,38 +201,21 @@ create view tenant_application_schemas as (
     ORDER BY app.tenant, app.name, configuration, schema
 );
 
-create view my_application_configurations as (
+create view my_email_template_defaults as (
     SELECT
         app.name as application,
-        ac.name as configuration,
-        ac.default_email_signup_template,
-        ac.default_email_already_signed_up_template,
-        ac.default_reset_password_template,
-        ac.url as url
+        etds.name as default_email_signup_template,
+        etdas.name as default_email_already_signed_up_template,
+        etdr.name as default_reset_password_template
     FROM
         application app
-    INNER JOIN application_configuration ac ON app.name = ac.application AND app.tenant = ac.tenant
+    LEFT JOIN email_template etds on app.tenant = etds.tenant and app.name = etds.application AND etds.default_email_signup
+    LEFT JOIN email_template etdas on app.tenant = etdas.tenant and app.name = etdas.application AND etdas.default_email_already_signed_up
+    LEFT JOIN email_template etdr on app.tenant = etdr.tenant and app.name = etdr.application AND etdr.default_reset_password
     WHERE app.tenant = get_tenant()
-    ORDER BY app.tenant, app.name, ac.name
+    ORDER BY app.name
 );
-GRANT SELECT ON my_application_configurations TO PUBLIC;
-
-create view my_application_schemas as (
-    SELECT
-        name as application,
-        configuration_name as configuration,
-        configuration_default_email_signup_template as default_email_signup_template,
-        configuration_default_email_already_signed_up_template as default_email_already_signed_up_template,
-        configuration_default_reset_password_template as default_reset_password_template,
-        configuration_url as url,
-        schema_name as schema,
-        schema_database as database,
-        schema_is_default as is_default
-    FROM
-        tenant_application_schemas
-    WHERE tenant = get_tenant()
-);
-GRANT SELECT ON my_application_schemas TO PUBLIC;
+GRANT SELECT ON my_email_template_defaults TO PUBLIC;
 
 create function add_email_account(_name varchar(60), _send_name varchar(255), _host varchar(255), _port integer, _username varchar(255), _encrypted_password text) returns void as
 $$
@@ -219,29 +224,41 @@ BEGIN
 END
 $$ language plpgsql SECURITY DEFINER;
 
-create function drop_email_account(_name varchar(60)) returns void as
+create function drop_email_account(_name varchar(60), _tenant postgres_addressable_name) returns void as
 $$
 BEGIN
-    DELETE FROM email_account WHERE tenant = get_tenant() AND name = _name;
+    DELETE FROM email_account WHERE tenant = _tenant AND name = _name;
 END
 $$ language plpgsql SECURITY DEFINER;
 
-create function register_email_template(_name varchar(60), _application postgres_addressable_name, _account varchar(60),
-                                        _description text, _app_relative_path postgres_addressable_name = null, _schema postgres_addressable_name = null,
+create function register_email_template(_name varchar(60), _application character varying(63), _subject varchar(255),
+                                        _description text, _app_relative_path safe_path = null, _schema character varying(63) = null,
                                         _data_validation_table postgres_addressable_name = null, _data_validation_view postgres_addressable_name = null, _recipient_validation_view postgres_addressable_name = null,
-                                        _allow_signup boolean = false, _allow_confirm_signup_attempt boolean = false, _allow_reset_password boolean = false) returns void as
+                                        _allow_signup boolean = false, _allow_confirm_signup_attempt boolean = false, _allow_reset_password boolean = false,
+                                        _default_email_signup boolean = null, _default_email_already_signed_up boolean = null, _default_reset_password boolean = null) returns void as
 $$
 BEGIN
+    if _default_email_signup is false then
+        _default_email_signup = null;
+    end if;
+    if _default_email_already_signed_up is false then
+        _default_email_already_signed_up = null;
+    end if;
+    if _default_reset_password is false then
+        _default_reset_password = null;
+    end if;
     INSERT INTO email_template(name, tenant, application,
-                               account, subject, description,
-                               app_relative_path, schema, data_validation_table,
-                               data_validation_view, recipient_validation_view, allow_signup,
-                               allow_confirm_signup_attempt, allow_reset_password) VALUES (
+                               subject, description, app_relative_path,
+                               schema, data_validation_table, data_validation_view,
+                               recipient_validation_view, allow_signup, allow_confirm_signup_attempt,
+                               allow_reset_password, default_email_signup, default_email_already_signed_up,
+                               default_reset_password) VALUES (
                                                                                            _name, get_tenant(), _application,
-                                                                                           _account, _description, _app_relative_path,
+                                                                                           _subject, _description, _app_relative_path,
                                                                                            _schema, _data_validation_table, _data_validation_view,
                                                                                            _recipient_validation_view, _allow_signup, _allow_confirm_signup_attempt,
-                                                                                           _allow_reset_password);
+                                                                                           _allow_reset_password, _default_email_signup, _default_email_already_signed_up,
+                                                                                           _default_reset_password);
 END
 $$ language plpgsql SECURITY DEFINER;
 
@@ -252,6 +269,23 @@ BEGIN
 END
 $$ language plpgsql SECURITY DEFINER;
 
+create function associate_email_template_with_application_configuration(_template varchar(60), _application character varying(63), _configuration varchar(63),
+                                                                        _account varchar(60), _override_send_name varchar(255) = null) returns void as
+$$
+BEGIN
+    INSERT INTO email_template_configuration(tenant, application, configuration,
+                                             template, account, override_send_name) VALUES (get_tenant(), _application, _configuration,
+                                                                                            _template, _account, _override_send_name);
+END
+$$ language plpgsql SECURITY DEFINER;
+
+create function disassociate_email_template_with_application_configuration(_template varchar(60), _application character varying(63), _configuration varchar(63)) returns void as
+$$
+BEGIN
+    DELETE FROM email_template_configuration WHERE tenant = get_tenant() AND application = _application AND template = _template AND configuration = _configuration;
+END
+$$ language plpgsql SECURITY DEFINER;
+
 -- tenant_database...
 alter table tenant_database add constraint tenant_database__tenant
     foreign key (tenant)
@@ -263,7 +297,7 @@ alter table account add constraint account__tenant
 -- account_password...
 alter table account_password add constraint account_password__account
     foreign key (account)
-        references account (user_id);
+        references account (user_id) ON DELETE CASCADE;
 -- application...
 alter table application add constraint application__tenant
     foreign key (tenant)
@@ -304,17 +338,15 @@ BEGIN
 END
 $$ language plpgsql;
 
-create function create_account_specifying_tenant(the_tenant postgres_addressable_name, enc_username text, _application postgres_addressable_name = null,
-                                                 _configuration postgres_addressable_name = null, _password text = null) returns jaaql_account_id as
+create function create_account_specifying_tenant(the_tenant postgres_addressable_name, enc_username text, _application character varying(63) = null,
+                                                 _configuration character varying(63) = null, _password text = null) returns jaaql_account_id as
 $$
 DECLARE
-    the_tenant postgres_addressable_name;
     account_id jaaql_account_id;
 BEGIN
-    SELECT get_tenant() INTO the_tenant;
     INSERT INTO account (tenant, username, application, configuration, public_password) VALUES (the_tenant, enc_username, _application, _configuration, _password) RETURNING user_id INTO account_id;
     EXECUTE 'CREATE ROLE ' || quote_ident(account_id);
-    if not is_public then
+    if _application is null then
         EXECUTE 'GRANT ' || the_tenant || '__user TO ' || quote_ident(account_id);
     end if;
     EXECUTE 'GRANT ' || the_tenant || '__public TO ' || quote_ident(account_id);
@@ -322,8 +354,8 @@ BEGIN
 END
 $$ language plpgsql;
 
-create function create_tenant_account(enc_username text, _application postgres_addressable_name = null,
-                                      _configuration postgres_addressable_name = null, _password text = null) returns jaaql_account_id as
+create function create_tenant_account(enc_username text, _application character varying(63) = null,
+                                      _configuration character varying(63) = null, _password text = null) returns jaaql_account_id as
 $$
 DECLARE
     account_id jaaql_account_id;
@@ -371,24 +403,24 @@ BEGIN
 END
 $$ language plpgsql SECURITY DEFINER;
 
-create function create_tenant_application(app_name postgres_addressable_name, app_description text) returns void as
+create function create_tenant_application(name postgres_addressable_name, description text) returns void as
 $$
 BEGIN
-    INSERT INTO application (tenant, description, name) VALUES (get_tenant(), app_description, app_name);
+    INSERT INTO application (tenant, description, name) VALUES (get_tenant(), create_tenant_application.description, create_tenant_application.name);
 END
 $$ language plpgsql SECURITY DEFINER;
 
-create function drop_tenant_application(app_name postgres_addressable_name) returns void as
+create function drop_tenant_application(name postgres_addressable_name) returns void as
 $$
 BEGIN
-    DELETE FROM application WHERE tenant = get_tenant() and name = app_name;
+    DELETE FROM application A WHERE tenant = get_tenant() and A.name = drop_tenant_application.name;
 END
 $$ language plpgsql SECURITY DEFINER;
 
-create function add_to_application_schema(app_name postgres_addressable_name, entry postgres_addressable_name, _is_default boolean = false) returns void as
+create function add_schema_to_application(application postgres_addressable_name, schema postgres_addressable_name, is_default boolean = false) returns void as
 $$
 BEGIN
-    INSERT INTO application_schema (tenant, application, name, is_default) VALUES (get_tenant(), app_name, entry, _is_default);
+    INSERT INTO application_schema (tenant, application, name, is_default) VALUES (get_tenant(), add_schema_to_application.application, add_schema_to_application.schema, add_schema_to_application.is_default);
 END;
 $$ language plpgsql SECURITY DEFINER;
 
@@ -399,10 +431,10 @@ BEGIN
 END;
 $$ language plpgsql SECURITY DEFINER;
 
-create function associate_database_to_application_configuration(app_name postgres_addressable_name, config_name postgres_addressable_name, entry postgres_addressable_name, the_database postgres_addressable_name) returns void as
+create function associate_database_to_application_configuration(application postgres_addressable_name, configuration postgres_addressable_name, schema postgres_addressable_name, database postgres_addressable_name) returns void as
 $$
 BEGIN
-    INSERT INTO application_database (tenant, application, configuration, schema, database) VALUES (get_tenant(), app_name, config_name, entry, the_database);
+    INSERT INTO application_database (tenant, application, configuration, schema, database) VALUES (get_tenant(), associate_database_to_application_configuration.application, associate_database_to_application_configuration.configuration, associate_database_to_application_configuration.schema, associate_database_to_application_configuration.database);
 END;
 $$ language plpgsql SECURITY DEFINER;
 
@@ -413,10 +445,10 @@ BEGIN
 END;
 $$ language plpgsql SECURITY DEFINER;
 
-create function create_application_configuration(app_name postgres_addressable_name, config_name postgres_addressable_name, _url text default null) returns void as
+create function create_application_configuration(application postgres_addressable_name, configuration postgres_addressable_name, url text, artifacts text) returns void as
 $$
 BEGIN
-    INSERT INTO application_configuration (tenant, application, url, name) VALUES (get_tenant(), app_name, _url, config_name);
+    INSERT INTO application_configuration (tenant, application, url, name, artifact_base_uri) VALUES (get_tenant(), create_application_configuration.application, create_application_configuration.url, create_application_configuration.configuration, artifacts);
 END;
 $$ language plpgsql SECURITY DEFINER;
 
@@ -427,13 +459,6 @@ BEGIN
 END;
 $$ language plpgsql SECURITY DEFINER;
 
-create function set_application_configuration_default_email_templates(_configuration varchar(63), _default_email_signup_template varchar(60), _default_email_already_signed_up_template varchar(60), _default_reset_password_template varchar(60)) returns void as
-$$
-BEGIN
-    UPDATE application_configuration SET default_email_signup_template = _default_email_signup_template, default_email_already_signed_up_template = _default_email_already_signed_up_template, default_reset_password_template = _default_reset_password_template WHERE tenant = get_tenant() AND name = _configuration;
-END;
-$$ language plpgsql SECURITY DEFINER;
-
 create function revoke_tenant_role_from_role(the_role postgres_addressable_name, from_role postgres_addressable_name) returns void as
 $$
 DECLARE
@@ -492,16 +517,18 @@ GRANT execute on function grant_tenant_role_to_account(postgres_addressable_name
 create table renderable_document (
     tenant    postgres_addressable_name not null,
     name      varchar(40) NOT null,
-    PRIMARY KEY (name, tenant),
+    application character varying(63) not null,
+    PRIMARY KEY (name, application, tenant),
+    FOREIGN KEY (tenant, application) REFERENCES application(tenant, name) ON UPDATE CASCADE ON DELETE CASCADE,
     url       text not null,
     render_as varchar(10) default 'pdf' not null,
     check (render_as in ('pdf'))
 );
 
-create function register_renderable_document(_name varchar(40), _url text, _render_as varchar(10)) returns void as
+create function register_renderable_document(_application character varying(63), _name varchar(40), _url text, _render_as varchar(10)) returns void as
 $$
 BEGIN
-    INSERT INTO renderable_document(name, tenant, url, render_as) VALUES (_name, get_tenant(), _url, _render_as);
+    INSERT INTO renderable_document(application, name, tenant, url, render_as) VALUES (_application, _name, get_tenant(), _url, _render_as);
 END
 $$ language plpgsql SECURITY DEFINER;
 
@@ -516,15 +543,16 @@ create table renderable_document_template (
     attachment varchar(40) not null,
     tenant postgres_addressable_name not null,
     template   varchar(60) not null,
-    PRIMARY KEY (attachment, template, tenant),
-    FOREIGN KEY (template, tenant) references email_template(name, tenant) ON DELETE CASCADE,
-    FOREIGN KEY (attachment, tenant) references renderable_document(name, tenant) ON DELETE CASCADE
+    application character varying(63) not null,
+    PRIMARY KEY (attachment, template, tenant, application),
+    FOREIGN KEY (template, tenant, application) references email_template(name, tenant, application) ON DELETE CASCADE,
+    FOREIGN KEY (attachment, tenant, application) references renderable_document(name, tenant, application) ON DELETE CASCADE
 );
 
-create function associate_renderable_document_with_template(_name varchar(40), _url text, _render_as varchar(10)) returns void as
+create function associate_renderable_document_with_template(_application character varying(63), _attachment varchar(40), _template varchar(60)) returns void as
 $$
 BEGIN
-    INSERT INTO renderable_document(name, tenant, url, render_as) VALUES (_name, get_tenant(), _url, _render_as);
+    INSERT INTO renderable_document_template(attachment, tenant, template, application) VALUES (_attachment, get_tenant(), _template, _application);
 END
 $$ language plpgsql SECURITY DEFINER;
 
@@ -535,10 +563,14 @@ BEGIN
 END
 $$ language plpgsql SECURITY DEFINER;
 
-create function create_tenant(the_tenant postgres_addressable_name, super_role jaaql_account_id = null) returns jaaql_account_id as
+create view check_drop_database as (SELECT true);
+create view check_refresh_application_config as (SELECT true);
+create view check_drop_email_account as (SELECT true);
+
+create function create_tenant(the_tenant postgres_addressable_name, _enc_symmetric_key varchar(255), super_role jaaql_account_id = null) returns jaaql_account_id as
 $$
 BEGIN
-    INSERT INTO tenant (name) VALUES (the_tenant);
+    INSERT INTO tenant (name, enc_symmetric_key) VALUES (the_tenant, _enc_symmetric_key);
 
     if super_role is null then
         SELECT gen_random_uuid() INTO super_role;
@@ -551,7 +583,9 @@ BEGIN
 
     EXECUTE 'CREATE ROLE ' || the_tenant || '__public';
     EXECUTE 'CREATE ROLE ' || the_tenant || '__user';
-    EXECUTE 'CREATE ROLE ' || the_tenant || '__admin';
+    if the_tenant <> 'jaaql' then
+        EXECUTE 'CREATE ROLE ' || the_tenant || '__admin';
+    end if;
     EXECUTE 'CREATE ROLE ' || the_tenant || '__super';
     EXECUTE 'GRANT ' || the_tenant || '__super' || ' to "' || super_role::text || '" with admin option';
     EXECUTE 'GRANT ' || the_tenant || '__admin' || ' to ' || the_tenant || '__super' || ' with admin option';
@@ -561,29 +595,31 @@ BEGIN
     EXECUTE 'GRANT ' || the_tenant || '__public' || ' to ' || the_tenant || '__admin' || ' with admin option';
 
     EXECUTE 'GRANT CONNECT ON DATABASE "jaaql__jaaql" TO "' || the_tenant || '__public"';
+    EXECUTE 'GRANT select on check_drop_database to ' || the_tenant || '__admin' || ' with grant option';
+    EXECUTE 'GRANT select on check_refresh_application_config to ' || the_tenant || '__admin' || ' with grant option';
+    EXECUTE 'GRANT select on check_drop_email_account to ' || the_tenant || '__admin' || ' with grant option';
 
     EXECUTE 'GRANT execute on function account_id_from_username(text) to ' || the_tenant || '__admin with grant option';
-    EXECUTE 'GRANT execute on function set_application_configuration_default_email_templates to ' || the_tenant || '__admin with grant option';
+    EXECUTE 'GRANT execute on function associate_email_template_with_application_configuration to ' || the_tenant || '__admin with grant option';
+    EXECUTE 'GRANT execute on function disassociate_email_template_with_application_configuration to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function register_renderable_document to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function deregister_renderable_document to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function associate_renderable_document_with_template to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function disassociate_renderable_document_from_template to ' || the_tenant || '__admin with grant option';
-    EXECUTE 'GRANT execute on function create_tenant_account(text, postgres_addressable_name, postgres_addressable_name, text) to ' || the_tenant || '__admin with grant option';
+    EXECUTE 'GRANT execute on function create_tenant_account to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function close_tenant_account(text) to ' || the_tenant || '__admin with grant option';
-    EXECUTE 'GRANT execute on function create_tenant_database(postgres_addressable_name, boolean) to ' || the_tenant || '__admin with grant option';
-    EXECUTE 'GRANT execute on function drop_tenant_database(postgres_addressable_name) to ' || the_tenant || '__admin with grant option';
+    EXECUTE 'GRANT execute on function create_tenant_database(postgres_addressable_name, boolean, boolean) to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function create_tenant_role(postgres_addressable_name) to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function drop_tenant_role(postgres_addressable_name) to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function create_tenant_application(postgres_addressable_name, text) to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function drop_tenant_application(postgres_addressable_name) to ' || the_tenant || '__admin with grant option';
-    EXECUTE 'GRANT execute on function create_application_configuration(postgres_addressable_name, postgres_addressable_name, text) to ' || the_tenant || '__admin with grant option';
+    EXECUTE 'GRANT execute on function create_application_configuration to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function drop_application_configuration(postgres_addressable_name, postgres_addressable_name) to ' || the_tenant || '__admin with grant option';
-    EXECUTE 'GRANT execute on function add_to_application_schema(app_name postgres_addressable_name, entry postgres_addressable_name, _is_default boolean) to ' || the_tenant || '__admin with grant option';
+    EXECUTE 'GRANT execute on function add_schema_to_application(app_name postgres_addressable_name, entry postgres_addressable_name, _is_default boolean) to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function drop_from_application_schema(postgres_addressable_name, postgres_addressable_name) to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function associate_database_to_application_configuration(postgres_addressable_name, postgres_addressable_name, postgres_addressable_name, postgres_addressable_name) to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function dissociate_database_from_application_configuration(postgres_addressable_name, postgres_addressable_name, postgres_addressable_name, postgres_addressable_name) to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function add_email_account to ' || the_tenant || '__admin with grant option';
-    EXECUTE 'GRANT execute on function drop_email_account to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function register_email_template to ' || the_tenant || '__admin with grant option';
     EXECUTE 'GRANT execute on function deregister_email_template to ' || the_tenant || '__admin with grant option';
 
@@ -591,14 +627,15 @@ BEGIN
 END
 $$ language plpgsql;
 
-create function setup() returns jaaql_account_id as
+create function setup(jaaql_enc_symmetric_key varchar(256), default_enc_symmetric_key varchar(256)) returns jaaql_account_id as
 $$
 DECLARE
     account_id jaaql_account_id;
 BEGIN
-    PERFORM create_tenant('jaaql', 'jaaql');
+    PERFORM create_tenant('jaaql', jaaql_enc_symmetric_key);
     GRANT jaaql__super to postgres with admin option;
-    SELECT create_tenant('default') INTO account_id;
+    GRANT jaaql__super to jaaql with admin option;
+    SELECT create_tenant('default', default_enc_symmetric_key) INTO account_id;
 
     UPDATE singleton SET override_tenant = 'jaaql';
     PERFORM create_tenant_database('jaaql', true);
@@ -606,13 +643,13 @@ BEGIN
     PERFORM create_tenant_application('console', 'The console application');
 
     PERFORM create_tenant_application('manager', 'The administration panel for JAAQL');
-    PERFORM add_to_application_schema('manager', 'default');
-    PERFORM create_application_configuration('manager', 'main');
+    PERFORM add_schema_to_application('manager', 'default');
+    PERFORM create_application_configuration('manager', 'main', '', '');
     PERFORM associate_database_to_application_configuration('manager', 'main', 'default', 'jaaql');
 
     PERFORM create_tenant_application('playground', 'Allows testing for new JAAQL/JEQL features');
-    PERFORM add_to_application_schema('playground', 'default');
-    PERFORM create_application_configuration('playground', 'main');
+    PERFORM add_schema_to_application('playground', 'default');
+    PERFORM create_application_configuration('playground', 'main', '', '');
     PERFORM associate_database_to_application_configuration('playground', 'main', 'default', 'play_db');
 
     return account_id;
@@ -625,7 +662,7 @@ create table account_ip (
     encrypted_address varchar(255),
     first_login timestamptz not null default current_timestamp,
     most_recent_login timestamptz not null default current_timestamp,
-    FOREIGN KEY (account) REFERENCES account,
+    FOREIGN KEY (account) REFERENCES account ON DELETE CASCADE,
     constraint user_ip_unq unique (encrypted_address)  -- Technically unique (user, address) but we are encrypting the ip in such a way that it's unique for the user
 );
 
@@ -639,7 +676,7 @@ create table log (
     login_ip uuid not null,
     status int not null,
     endpoint varchar(64) not null,
-    FOREIGN KEY (the_user) REFERENCES account,
+    FOREIGN KEY (the_user) REFERENCES account ON DELETE CASCADE,
     FOREIGN KEY (login_ip) REFERENCES account_ip
 );
 
@@ -679,16 +716,9 @@ create view fetch_recent_passwords as (
         a.username,
         password_hash
     FROM
-         account_password ap
+         account a
     INNER JOIN
-        account a on ap.account = a.user_id
-    WHERE
-        (account, created) in
-            (SELECT
-                account,
-                max(created)
-            FROM account_password
-            GROUP BY account)
+        account_password ap on ap.id = a.password
 );
 
 create view fetch_recent_passwords_with_ips as (
@@ -713,7 +743,7 @@ create table sign_up (
     activated boolean not null default false,
     used_key_a boolean not null default false,
     account jaaql_account_id not null,
-    FOREIGN KEY (account) REFERENCES account,
+    FOREIGN KEY (account) REFERENCES account ON DELETE CASCADE,
     tenant postgres_addressable_name not null,
     closed timestamptz,
     created timestamptz default current_timestamp not null,
@@ -721,7 +751,10 @@ create table sign_up (
     code_expiry_ms integer not null default 1000 * 60 * 15, -- 15 minutes
     email_template varchar(60),
     configuration varchar(63),
-    FOREIGN KEY (email_template, tenant) REFERENCES email_template (name, tenant),
+    application character varying(63),
+    check ((application is null) = (configuration is null)),
+    FOREIGN KEY (email_template, tenant, application) REFERENCES email_template (name, tenant, application) ON DELETE CASCADE ON UPDATE CASCADE,
+    FOREIGN KEY (email_template, tenant, application, configuration) REFERENCES email_template_configuration (template, tenant, application, configuration) ON DELETE CASCADE ON UPDATE CASCADE,
     data_lookup_json text,
     check ((data_lookup_json is null and email_template is null) or email_template is not null)
 );
@@ -735,15 +768,19 @@ create table reset_password (
     activated boolean not null default false,
     used_key_a boolean not null default false,
     the_user jaaql_account_id not null,
-    FOREIGN KEY (the_user) REFERENCES account,
+    FOREIGN KEY (the_user) REFERENCES account ON DELETE CASCADE,
     closed timestamptz,
     created timestamptz default current_timestamp not null,
     expiry_ms integer not null default 1000 * 60 * 60 * 2, -- 2 hours. Important this is the same as above fake table
     code_expiry_ms integer not null default 1000 * 60 * 15, -- 15 minutes. Important this is the same as above fake table
     email_template varchar(60),  -- Deliberately null as we can have a reset password managed by the admin
+    application character varying(63),
+    configuration varchar(63),
+    check ((application is null) = (configuration is null)),
     tenant postgres_addressable_name not null,
     data_lookup_json text,
-    FOREIGN KEY (email_template, tenant) REFERENCES email_template(name, tenant)
+    FOREIGN KEY (email_template, tenant, application) REFERENCES email_template(name, tenant, application) ON DELETE CASCADE ON UPDATE CASCADE,
+    FOREIGN KEY (email_template, tenant, application, configuration) REFERENCES email_template_configuration (template, tenant, application, configuration) ON DELETE CASCADE ON UPDATE CASCADE
 );
 
 create table fake_reset_password (
@@ -760,9 +797,11 @@ create table email_history (
     id uuid PRIMARY KEY NOT NULL default gen_random_uuid(),
     template varchar(60) not null,
     tenant postgres_addressable_name not null,
-    FOREIGN KEY (template, tenant) REFERENCES email_template(name, tenant),
+    application character varying(63) not null,
+    override_send_name varchar(255),
+    FOREIGN KEY (template, tenant, application) REFERENCES email_template(name, tenant, application) ON DELETE SET NULL,
     sender jaaql_account_id not null,
-    FOREIGN KEY (sender) REFERENCES account,
+    FOREIGN KEY (sender) REFERENCES account ON DELETE CASCADE,
     sent timestamptz not null default current_timestamp,
     encrypted_subject text,
     encrypted_recipients text not null,
@@ -775,11 +814,15 @@ create table rendered_document (
     document_id uuid PRIMARY KEY not null default gen_random_uuid(),
     document varchar(40) NOT NULL,
     tenant postgres_addressable_name not null,
+    application character varying(63) not null,
+    configuration character varying(63) not null,
+    FOREIGN KEY (application, tenant) REFERENCES application(name, tenant) ON DELETE CASCADE,
+    FOREIGN KEY (application, configuration, tenant) REFERENCES application_configuration (application, name, tenant),
     created timestamptz not null default current_timestamp,
     encrypted_parameters text,
     encrypted_access_token text,
     create_file boolean not null,
-    FOREIGN KEY (document, tenant) REFERENCES renderable_document(name, tenant),
+    FOREIGN KEY (document, tenant, application) REFERENCES renderable_document(name, tenant, application) ON DELETE CASCADE,
     completed timestamptz,
     content bytea,
     filename varchar(100),
