@@ -100,7 +100,7 @@ def encrypt_raw(secret_key: bytes, data: any, iv: bytes = None) -> str:
     return b64e(iv).decode(ENCODING__ascii) + "." + b64e(ct).decode(ENCODING__ascii)
 
 
-def decrypt__raw(secret_key: bytes, data: str) -> str:
+def decrypt_raw(secret_key: bytes, data: str) -> str:
     unpadder = padding.PKCS7(PKCS7__length).unpadder()
     iv = b64d(data.split(".")[0])
     data = b64d(data.split(".")[1])
